@@ -51,6 +51,86 @@ list_fevd = function(lst,evd_mod_str,nsloc=NULL,outfile=NULL,pc_complete=0.8,min
 }
 
 
+#' Fit EVD Parameters to Rows of a Data Frame of Annual Maxima
+#'
+#' @description
+#' Fits an extreme value distribution (EVD) to each row of a \code{data.frame}, where each row represents a time series of annual maxima. Optionally includes non-stationary covariates in the location parameter. Parallel processing is used to improve efficiency.
+#'
+#' @param varbIN A numeric \code{data.frame} where each row is a vector of annual maxima values (e.g., one row per location or site).
+#' @param cores An \code{integer} specifying the number of parallel cores to use for computation.
+#' @param ntries The \code{integer} number of tries to fit the evd
+#' @param evd_mod_str A \code{character} string specifying the EVD fitting function to use. Must be one of \code{"fgumbel"}, \code{"fgumbelx"}, or \code{"fgev"}.
+#' @param nsloc Optional \code{data.frame} of covariates for non-stationary location modelling. Must have the same number of rows as \code{varbIN}.
+#' @param seed Optional \code{integer} for reproducibility. If supplied, a random seed will be set before model fitting.
+#'
+#' @details
+#' Each row of \code{varbIN} is passed to \code{evd_params()}, optionally with a corresponding row of covariates from \code{nsloc}. Covariates are internally standardised if provided. The function constructs a blank parameter structure (\code{empty_evd_params}) to ensure consistent output structure, even if fitting fails.
+#'
+#' Parallel computation is performed using \code{\link[parallel]{parLapply}}.
+#'
+#' @return A \code{data.frame} with one row per input time series, containing the fitted EVD parameters returned by \code{evd_params()}.
+#'
+#' @seealso
+#' \code{\link{list_fevd}} to apply EVD fitting to a list of annual maxima,
+#' \code{\link{evd_params}} for direct fitting to a single time series.
+#' #'
+#' @examples
+#' set.seed(42)
+#' df <- as.data.frame(matrix(evd::rgev(1000), nrow = 100))
+#' results <- df_fevd(df, cores = 1, evd_mod_str = "fgumbel")
+#'
+#' @export
+df_fevd <- function(varbIN,cores=1,ntries=1,evd_mod_str = "fgumbel",nsloc=NULL,seed=NULL){
+  if(!is.null(nsloc)){
+    rawnsloc <- nsloc
+    nsloc <- centredAndScaled(nsloc)
+  }
+  cl <- parallel::makeCluster(cores)
+  S1 <- Sys.time()
+  # Export necessary variables and functions to the cluster
+
+  print(evd_mod_str)
+
+  if (!is.null(seed)) {
+    set.seed(seed)
+  }
+
+  uvdata <- evd::rgev(100, loc = 0.13, scale = 1.1, shape = 0.2)
+  if (is.null(nsloc))
+    empty_evd_params <- evd_params(uvdata, evd_mod_str) *
+    NA
+  if (!is.null(nsloc)) {
+    nns = dim(nsloc)[2]
+    uvnsloc <- as.data.frame(array(stats::rnorm(1000), dim = c(100,
+                                                               nns)))
+    names(uvnsloc) <- names(nsloc)
+    empty_evd_params <- evd_params(x = uvdata, evd_mod_str = evd_mod_str,
+                                    nsloc = uvnsloc, returncs = FALSE, silent = FALSE) *
+      NA
+  }
+
+  nvarb <- nrow(varbIN)
+  varbIN <- varbIN
+  evd_mod_str <- evd_mod_str
+
+  if( is.null(nsloc)) parallel::clusterExport(cl, varlist = c("varbIN", "evd_params","evd_mod_str","empty_evd_params"), envir = environment())
+
+  if(!is.null(nsloc)) parallel::clusterExport(cl, varlist = c("varbIN", "evd_params","evd_mod_str","nsloc","rawnsloc","empty_evd_params"), envir = environment())
+
+  parallel::clusterEvalQ(cl, library(evd))
+
+  if( is.null(nsloc)) varb_evd_l <- parallel::parLapply(cl, 1:nvarb, function(x) evd_params(varbIN[x,], evd_mod_str = evd_mod_str,empty_evd_params=empty_evd_params,ntries=ntries))
+
+  if(!is.null(nsloc)) varb_evd_l <- parallel::parLapply(cl, 1:nvarb, function(x) evd_params(varbIN[x,], evd_mod_str = evd_mod_str,nsloc = rawnsloc,empty_evd_params=empty_evd_params,ntries=ntries))
+
+  parallel::stopCluster(cl)
+  print(Sys.time()-S1)
+
+  varb_evd = as.data.frame(do.call(rbind,varb_evd_l))
+  return(varb_evd)
+}
+
+
 #' Centre and Scale Numeric Data
 #'
 #' \code{centredAndScaled} centres (subtracts the mean) and scales (divides by
@@ -102,7 +182,7 @@ centredAndScaled = function(nsloc = NULL){
 #' @param nsloc A data frame with the same number of rows as the length of x, for linear modelling of the location parameter. The data frame is treated as a covariate matrix (excluding the intercept). A numeric vector can be given as an alternative to a single column data frame.
 #' @param outfile filename to write to netcdf
 #' @param cores positive integer. If cores > 1, a 'parallel' package cluster with that many cores is created and used. You can also supply a cluster object. Ignored for functions that are implemented by terra in C++ (see under fun)
-#' @param ntries integer number of attempts at fitting fgumbelx
+#' @param ntries The \code{integer} number of tries to fit the evd
 #' @param silent logical: should the report of error messages be suppressed?
 #' @param seed set the seed for fitting.
 #'
